/*
    SPDX-FileCopyrightText: 2007 Krzysztof Kundzicz <athantor@gmail.com>
    SPDX-License-Identifier: GPL-2.0-or-later
*/

#include <PluginPage.h>

namespace kt
{

PluginPage::PluginPage(QWidget* p) : QWidget(p)
{
}


PluginPage::~PluginPage()
{
}

} //ns end

#include "moc_PluginPage.cpp"
