<?xml version="1.0"?>
<!DOCTYPE gui SYSTEM "kpartgui.dtd">
<gui library="libktbwschedulerplugin" name="ktbwschedulerplugin" version="1">
	<ToolBar name="SchedulerToolbar">
		<text>Bandwidth Schedule</text>
		<Action name="schedule_load" />
		<Action name="schedule_save" />
		<Action name="schedule_clear" />
		<Separator/>
		<Action name="new_schedule_item" />
		<Action name="remove_schedule_item" />
		<Action name="edit_schedule_item" />
		<Separator/>
		<Action name="schedule_active" />
	</ToolBar>
</gui>
