<?xml version="1.0"?>
<!DOCTYPE gui SYSTEM "kpartgui.dtd">
<gui library="libktmediaplayerplugin" name="mediaplayerplugin" version="2">
<Menu name="MediaPlayerMenu"><text>Media Player Menu</text>
	<Action name="media_play"/>
	<Action name="media_pause"/>
	<Action name="media_stop"/>
	<Action name="media_next"/>
	<Action name="media_prev"/>
</Menu>

</gui>
