<?xml version="1.0"?>
<!DOCTYPE gui SYSTEM "kpartgui.dtd">
<gui library="libktmagnetgeneratorplugin" name="magnetgeneratorplugin" version="1">
	<Menu name="ViewMenu">
		<Action name="generate_magnet"/>
	</Menu>
</gui>
