<?xml version="1.0"?>
<!DOCTYPE gui SYSTEM "kpartgui.dtd">
<gui library="libktorrent_downloadorder" name="downloadorderplugin" version="1">
	<Menu name="ViewMenu">
		<Action name="download_order"/>
	</Menu>
</gui>
