/*
    SPDX-FileCopyrightText: 2005 Joris Guisson <joris.guisson@gmail.com>
    SPDX-License-Identifier: GPL-2.0-or-later
*/

#include "guiinterface.h"
#include <interfaces/torrentinterface.h>

namespace kt
{
GUIInterface::GUIInterface()
{
}

GUIInterface::~GUIInterface()
{
}
}
