/*
    SPDX-FileCopyrightText: 2005 Joris Guisson <joris.guisson@gmail.com>
    SPDX-License-Identifier: GPL-2.0-or-later
*/

#include "coreinterface.h"

namespace kt
{
CoreInterface::CoreInterface()
{
}

CoreInterface::~CoreInterface()
{
}
}

#include "moc_coreinterface.cpp"
